/*
 This file and all of its content are part of Q library
 by Jan Maska (http://janmaska.com), and published under
 Creative Commons CC BY-NC-SA License.
 For full license, go to https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode
 */

if(typeof(Q)==='undefined') { Q = {}; }

Q.__Stack = [];
Q.options = {
	language:	'en',
	loggingMinimumPriority:	1
};
Q.loggingMinimumPriority = 0;


Q.AddStack = function()
{
	var message	    	= (typeof(arguments[0])==='undefined') ? false:(typeof(arguments[0])==='object') ? arguments[0]:{message:arguments[0],type:'info',origin:false,time:false,arguments:false,element:false};
	if(!message) 		{ return false;}
	message.time    	= new Date().getTime();
	message.type    	= message.type.toLowerCase();
	message.priority 	= (typeof(message.priority)!=='undefined')?message.priority:0;
	
	if(message.priority >= Q.options.loggingMinimumPriority)
	{
		this.__Stack.push(message);
	}
	return this.__Stack.length;
};

Q.GetStack = function()
{
	var filter 		= (typeof(arguments[0])==='undefined') ? false:arguments[0];
	var result		= [];
	var match		= true;
	var i=0; do
	{
		
		if(typeof(filter.origin)!=='undefined') 	{ match =(this.__Stack[i].origin.toString().indexOf(filter.origin)!==-1) ? true:false; }
		if(typeof(filter.type)!=='undefined') 		{ match =(filter.type.toLowerCase()===this.__Stack[i].type.toLowerCase()) ? true:false; }
		if(typeof(filter.message)!=='undefined') 	{ match =(this.__Stack[i].message.toString().indexOf(filter.message)!==-1) ? true:false; }
		if(typeof(filter.instance)!=='undefined') 	{ match =(this.__Stack[i].instance.toString().indexOf(filter.instance)!==-1) ? true:false; }
		if(match) { result.push(this.__Stack[i]); }
		i++;
	} while((typeof(this.__Stack[i])!=='undefined'));
	return result;
};

Q.Template = function()
{
	var	result='',source,map,cache,
		template	= (typeof(arguments[0])==='undefined') ? false:arguments[0],
		variables 	= (typeof(arguments[1])==='undefined') ? false:(!arguments[1])?[]:($.isArray(arguments[1]))?arguments[1]:(arguments[1] instanceof $)?arguments[1]:[arguments[1]],
		filter		= (typeof(arguments[2])==='undefined' || !arguments[2]) ? function(){return true;}:arguments[2],
		templateNS 	= (typeof(arguments[3])==='undefined') ? false:arguments[3];
	
	Q.AddStack({message:'Loading template',type:'Info',origin:'Q.Template()',arguments:arguments,element:$(template)});
	
	if(!template)	{ Q.AddStack({message:'Template ID not provided!',type:'error',origin:'Q.Template()',arguments:arguments,element:$(template)}); return false; }
	source			= $('#'+template+'[data-content="QTemplate"]');
	if(!source[0])	{ Q.AddStack({message:'Template source not found!',type:'error',origin:'Q.Template()',arguments:arguments,element:$(template)}); return false; }
	source			= source.html();
	if(source==='')	{ Q.AddStack({priority:1,message:'Template empty!',type:'error',origin:'Q.Template()',arguments:arguments,element:$(template)}); return false; }
	
	if(!variables)	
	{
		Q.AddStack({message:'No variables to parse',type:'Warning',origin:'Q.Template()',arguments:arguments,element:$(template)});
		return source;
	}
	
	var i=0; do
	{
		if(filter.call(this,variables[i]))
		{
			map		= $.map(variables[i],function(value,key){return key;});
			cache	= source;
			
			for(n in map){
				if(typeof(variables[i][map[n]])==='array'  || typeof(variables[i][map[n]])==='object') 
				{
					var childNS = (templateNS) ? templateNS+'.'+map[n]:false,
						childTN = (childNS) ? childNS:map[n];
					
					cache = cache.replace(new RegExp('\{Q\:'+childTN+'\}','gm'), Q.Template(template+'\\.'+map[n],variables[i][map[n]],filter,childNS));
					
				}
				else {
					try
					{
						cache = cache.replace(new RegExp('\{Q\:'+map[n]+'\}','gm'),variables[i][map[n]]);
						cache = cache.replace(new RegExp('\{Q\:'+map[n]+'\:lower\}','gm'),variables[i][map[n]].toString().toLowerCase());
						cache = cache.replace(new RegExp('\{Q\:'+map[n]+'\:upper\}','gm'),variables[i][map[n]].toString().toUpperCase());
					} catch(e) { Q.AddStack({message:e,type:'Warning',origin:'Q.Template()',arguments:arguments,element:$(template)}); }
				}
			}
			result += cache;
		}
	} while(i++,i<variables.length);
	
	Q.AddStack({message:'Template parsed OK',type:'Info',origin:'Q.Template()',arguments:arguments,element:$(template)});
	return result;
};

Q.GUID = function()
{
	return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
		var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
		return v.toString(16);
	});
};

Q.UXTimeParse = function()
{
	var inputDate = (typeof(arguments[0])!=='undefined') ? arguments[0]:new Date().getTime();
	if(arguments[0]==0) { return "unknown"; }
	var d = new Date(inputDate * 1000);
	var now = new Date();
	var startOfDay = new Date(now.getFullYear(), now.getMonth(), now.getDate());
	var startOfDayTS = startOfDay.getTime();
	var result='';
	var hrs = (d.getHours().toString().length < 2) ? '0'+d.getHours():d.getHours();
	var min = (d.getMinutes().toString().length < 2) ? '0'+d.getMinutes():d.getMinutes();
	
	if(d < (startOfDayTS - 172800) ) // older than two days
	{
		result = d.getMonth()+'/'+d.getDate()+'/'+d.getFullYear()+' '+Q.lang[Q.options.language].datetime.At+' '+hrs+':'+min;
	}
	else if(d < (startOfDayTS - 86400) ) // two days ago
	{
		result = Q.lang[Q.options.language].datetime.TwoDaysAgo+' '+Q.lang[Q.options.language].datetime.At+' '+hrs+':'+min;
	}
	else if(d < (startOfDayTS) ) // yesterday
	{
		result = Q.lang[Q.options.language].datetime.Yesterday+' '+Q.lang[Q.options.language].datetime.At+' '+hrs+':'+min;
	}
	else // today
	{
		result = Q.lang[Q.options.language].datetime.Today+' '+Q.lang[Q.options.language].datetime.At+' '+hrs+':'+min;
	}
	
	return result;
};

Q.eventQueue = {
	onInit: 		[],
	onReady:		[],
	onDestroy:		[]
};

Q.eventTrigger =		function(e)
{
	var data = (typeof(arguments[1])!=='undefined') ? arguments[1]:false;
	if(typeof(Q.eventQueue[e])=='undefined')
	{
		Q.AddStack({priority:1,message:'Attempting to trigger nonexistent event "'+e+'"',type:'Error',origin:'Q.eventTrigger'});
		return Q;
	}
	for(var i in Q.eventQueue[e]) { Q.eventQueue[e][i].call(this,data); }
	return Q;
};

Q.dropEvent =			function()
{
	if(typeof(arguments[0])=='undefined') { return Q; }
	if(typeof(arguments[1])!=='undefined') 
	{
		var newE = [],fc,cmp;
		for(var i=0;i<Q.eventQueue[arguments[0]].length;i++)
		{
			fc	= Q.eventQueue[arguments[0]][i].toString();cmp	= arguments[1].toString();
			if(fc!==cmp) { newE.push(Q.eventQueue[arguments[0]][i]); }
		}
		Q.eventQueue[arguments[0]] = newE; return Q;
	}
	Q.eventQueue[arguments[0]] = [];
	return Q;
};

Q.onInit =				function() { if(typeof(arguments[0])==='undefined') {return Q.eventQueue['onInit'];}	Q.eventQueue.onInit.push(arguments[0]); 	return Q; },
Q.onReady =				function() { if(typeof(arguments[0])==='undefined') {return Q.eventQueue['onReady'];}	Q.eventQueue.onReady.push(arguments[0]); 	return Q; },
Q.onDestroy =			function() { if(typeof(arguments[0])==='undefined') {return Q.eventQueue['onDestroy'];} Q.eventQueue.onDestroy.push(arguments[0]); 	return Q; }

Q.HTML = {
	encode: function()
	{
		if(typeof(arguments[0])==='undefined') { return Q; }
		return $('<div/>').text(arguments[0]).html();
	},
	decode: function()
	{
		if(typeof(arguments[0])==='undefined') { return Q; }
		return $('<div/>').html(arguments[0]).text();
	}
};

Q.Base64 = {

	// private property
	_keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

	// public method for encoding
	encode : function (input) {
		var output = "";
		var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
		var i = 0;

		input = Q.Base64._utf8_encode(input);

		while (i < input.length) {
			chr1 = input.charCodeAt(i++);
			chr2 = input.charCodeAt(i++);
			chr3 = input.charCodeAt(i++);

			enc1 = chr1 >> 2;
			enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
			enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
			enc4 = chr3 & 63;

			if (isNaN(chr2)) {
				enc3 = enc4 = 64;
			} else if (isNaN(chr3)) {
				enc4 = 64;
			}

			output = output +
			this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) +
			this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);
		}
		return output;
	},

	// public method for decoding
	decode : function (input) {
		var output = "";
		var chr1, chr2, chr3;
		var enc1, enc2, enc3, enc4;
		var i = 0;

		input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

		while (i < input.length) {
			enc1 = this._keyStr.indexOf(input.charAt(i++));
			enc2 = this._keyStr.indexOf(input.charAt(i++));
			enc3 = this._keyStr.indexOf(input.charAt(i++));
			enc4 = this._keyStr.indexOf(input.charAt(i++));

			chr1 = (enc1 << 2) | (enc2 >> 4);
			chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
			chr3 = ((enc3 & 3) << 6) | enc4;

			output = output + String.fromCharCode(chr1);

			if (enc3 != 64) {
				output = output + String.fromCharCode(chr2);
			}
			if (enc4 != 64) {
				output = output + String.fromCharCode(chr3);
			}
		}
		output = Q.Base64._utf8_decode(output);
		return output;
	},

	// private method for UTF-8 encoding
	_utf8_encode : function (string) {
		string = string.replace(/\r\n/g,"\n");
		var utftext = "";

		for (var n = 0; n < string.length; n++) {
			var c = string.charCodeAt(n);

			if (c < 128) {
				utftext += String.fromCharCode(c);
			}
			else if((c > 127) && (c < 2048)) {
				utftext += String.fromCharCode((c >> 6) | 192);
				utftext += String.fromCharCode((c & 63) | 128);
			}
			else {
				utftext += String.fromCharCode((c >> 12) | 224);
				utftext += String.fromCharCode(((c >> 6) & 63) | 128);
				utftext += String.fromCharCode((c & 63) | 128);
			}
		}
		return utftext;
	},

	// private method for UTF-8 decoding
	_utf8_decode : function (utftext) {
		var string = "";
		var i = 0;
		var c = c1 = c2 = 0;

		while ( i < utftext.length ) {
			c = utftext.charCodeAt(i);
			if (c < 128) {
				string += String.fromCharCode(c);
				i++;
			}
			else if((c > 191) && (c < 224)) {
				c2 = utftext.charCodeAt(i+1);
				string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
				i += 2;
			}
			else {
				c2 = utftext.charCodeAt(i+1);
				c3 = utftext.charCodeAt(i+2);
				string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
				i += 3;
			}
		}
		return string;
	}
}

// Extend jQuery features
$.fn.getType = function(){ return this[0].tagName == "INPUT" ? this[0].type.toLowerCase() : this[0].tagName.toLowerCase(); }