/*
	This file and all of its content are part of Q library
	by Jan Maska (http://janmaska.com), and published under
	Creative Commons CC BY-NC-SA License.
	For full license, go to https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode
*/

// EXTEND JQUERY EVENT MODEL
(function($){$.event.special.destroyed={remove:function(o){if(o.handler&&o.type!=='destroyed'){o.handler();}}};})(jQuery);

Q.PluginFactory = {};

Q.PluginFactory.Plugin = function()
{
	return function(el,user_options){if(el){this.__init(el,user_options);}};
};

Q.PluginFactory.Register = function(plugin)
{
	$.fn[plugin.prototype.name] = function(options)
	{
		var args = $.makeArray(arguments), after = args.slice(1);
		
		if(typeof(options)==="string")
		{
			return this.map(function()
			{
				var instance = $.data(this, plugin.prototype.name), res = false;
				if(instance)
				{
					res 	= instance[options].apply(instance,after);
					if(res)	{ return res; }
					return	instance;
				}
				return false;
			}).get();
		}
		
		return this.each(function()
		{
			var instance	= $.data(this, plugin.prototype.name);
			if(instance)	{ if(instance.update){ instance.update.apply(instance,args); } }
			else			{ return $.data(this, plugin.prototype.name, new plugin(this,options)); }
		});
	};
};


/*
	BEGIN: BASE PROTOTYPE OF A PLUGIN
*/

Q.PluginFactory.Base = Q.PluginFactory.Plugin();
Q.PluginFactory.Register(Q.PluginFactory.Base);

$.extend(Q.PluginFactory.Base.prototype,{
	name:				'Q.PluginFactory.Base',
	id:					false,
	options:			{},
	super:				false,
	cache:				{},
	default_options:	{
							super:	false,
							bubbleEvents:{}
						}
});

$.extend(Q.PluginFactory.Base.prototype,{
	__init: function(el,user_options)
	{
		this.$e = $.call(this,el);
		this.id = this.$e.attr('id');
		if(typeof(this.id)==='undefined') { this.id = Q.GUID(); this.$e.attr('id',this.id); }
		$.extend(this.options,this.default_options,user_options);
		
		if(typeof(this.options.super)!=='undefined')	{ this.super = this.options.super; }
		if(typeof(this.options.onInit)!=='undefined')	{ this.onInit(this.options.onInit); }
		if(typeof(this.options.onReady)!=='undefined')	{ this.onReady(this.options.onReady); }
		if(typeof(this.options.onDestroy)!=='undefined'){ this.onDestroy(this.options.onDestroy); }
		
		this.eventTrigger('onInit',{instance:this});
		if(this.options.bubbleEvents.onInit) { this.super.eventTrigger('onInit',{instance:this}); }
		
		Q.AddStack({priority:1,message:'Instance of '+this.name+' initialized',type:'Info',origin:this.name+'::__init()',arguments:arguments,element:el,instance:this.id});
		
		this.init(el,user_options);
		this.$e.on('destroyed',{instance:this},function(e){e.data.instance.destroy();});
		this.eventTrigger('onReady',{instance:this});
		if(this.options.bubbleEvents.onReady) { this.super.eventTrigger('onReady',{instance:this}); }
	},
	init: function(el,user_options)
	{
	},
	destroy: function()
	{
		this.eventTrigger('onDestroy',{instance:this});
		if(this.options.bubbleEvents.onDestroy) { this.super.eventTrigger('onDestroy',{instance:this}); }
		this.$e.off(this.name);
		this.$e.unbind('.'+this.name);
		$.removeData(this.$e);
		Q.AddStack({priority:1,message:'Instance of '+this.name+' destroyed: ID '+this.id,type:'Info',origin:'__init()',arguments:arguments,instance:this.id});
	}
});

$.extend(Q.PluginFactory.Base.prototype,{
	eventQueue:
	{
		onInit:				[],
		onReady:			[],
		onDestroy:			[]
	},
	eventTrigger: 		function(e)
	{
		var data = (typeof(arguments[1])!=='undefined') ? arguments[1]:false;
		for(var i in this.eventQueue[e]) { this.eventQueue[e][i].call(this,data); }
	},
	dropEvent:			function()
	{
		if(typeof(arguments[0])!=='undefined') { return this; }
		if(typeof(arguments[1])!=='undefined') 
		{
			var newE = [],fc,cmp;
			for(var i=0;i<this.eventQueue[arguments[0]].length;i++)
			{
				fc	= this.eventQueue[arguments[0]][i].toString();cmp	= arguments[1].toString();
				if(fc!==cmp) { newE.push(this.eventQueue[arguments[0]][i]); }
			}
			this.eventQueue[arguments[0]] = newE;return this;
		}
		this.eventQueue[arguments[0]] = [];
	},
	onInit: 				function() { if(typeof(arguments[0])==='undefined') {return this.eventQueue['onInit'];}		this.eventQueue.onInit.push(arguments[0]); 		return this; },
	onReady:    			function() { if(typeof(arguments[0])==='undefined') {return this.eventQueue['onReady'];}	this.eventQueue.onReady.push(arguments[0]); 	return this; },
	onDestroy:	    		function() { if(typeof(arguments[0])==='undefined') {return this.eventQueue['onDestroy'];}  this.eventQueue.onDestroy.push(arguments[0]); 	return this; }
});

/*
	END: BASE PROTOTYPE OF A PLUGIN
*/

/*
	BEGIN: BASE PROTOTYPE OF A PLUGIN
*/

Q.PluginFactory.Dropdown = Q.PluginFactory.Plugin();
$.extend(Q.PluginFactory.Dropdown.prototype, Q.PluginFactory.Base.prototype);
Q.PluginFactory.Register(Q.PluginFactory.Dropdown);

$.extend(Q.PluginFactory.Dropdown.prototype,{
	name:					"Q.PluginFactory.Dropdown",
	default_options:		{
								data:[],
								endlessLoop:true,
								accessibility:{
									dialogInstructionsText: "Hit Enter key to change the status. You can use arrows to navigate the options. Use arrow down or right to move to the next option and arrow up or left to move to previous option. You make your selection by pressing the Enter key. To cancel the selection process, press Escape or Tab key.",
									lastItemInListText:     "This is the last option.",
									firstItemInListText:	"This is the first option.",
									lastItemEndlessText:	"Navigate to next item to get to the beginning of the list.",
									firstItemEndlessText:	"Navigate to previous item to get to the end of the list."
								},
								templates:{
									listItemTemplate:		false,
									triggerTemplate:		false
								}
							},
	selectedItem:			{label:false,id:false,meta:false},
	$trigger:				false,
	$list:					false,
	$gauntlet:				false
});


$.extend(true, Q.PluginFactory.Dropdown.prototype,{
	eventQueue:
	{
		onActivate:			[],
		onFirstActivate:	[],
		onSelect: 			[],
		onData:				[],
		onClose:			[]
	},
	onActivate: 			function() { if(typeof(arguments[0])==='undefined') {return this.eventQueue['onActivate'];} this.eventQueue.onActivate.push(arguments[0]); return this; },
	onFirstActivate: 		function() { if(typeof(arguments[0])==='undefined') {return this.eventQueue['onFirstActivate'];} this.eventQueue.onFirstActivate.push(arguments[0]); return this; },
	onSelect: 				function() { if(typeof(arguments[0])==='undefined') {return this.eventQueue['onSelect'];} this.eventQueue.onSelect.push(arguments[0]); return this; },
	onData: 				function() { if(typeof(arguments[0])==='undefined') {return this.eventQueue['onData'];} this.eventQueue.onData.push(arguments[0]); return this; },
	onClose: 				function() { if(typeof(arguments[0])==='undefined') {return this.eventQueue['onClose'];} this.eventQueue.onClose.push(arguments[0]); return this; }
});

$.extend(Q.PluginFactory.Dropdown.prototype,{
	__init:					function(el,user_options)
	{
		Q.PluginFactory.Base.prototype.__init.apply(this,arguments);
		
		if(typeof(this.options.onActivate)!=='undefined')		{ this.onActivate(this.options.onActivate); }
		if(typeof(this.options.onFirstActivate)!=='undefined')	{ this.onFirstActivate(this.options.onFirstActivate); }
		if(typeof(this.options.onSelect)!=='undefined')			{ this.onSelect(this.options.onSelect); }
		if(typeof(this.options.onData)!=='undefined')			{ this.onData(this.options.onData); }
		if(typeof(this.options.onClose)!=='undefined')			{ this.onClose(this.options.onClose); }
		
		this.__buildHTML();
		this.__registerHTML();
		this.__registerEvents();
	},
	
	getValue:				function()
	{
		return this.selectedItem;
	},
	
	__buildHTML:			function()
	{
		this.$e.addClass('dropdown-selector');
		var triggerHTML = Q.Template(this.options.templates.triggerTemplate);
		if(!triggerHTML)
		{
			Q.AddStack({message:'Defaulting to native output',type:'info',origin:this.name+'::__buildHTML()',arguments:arguments,instance:this.id});
			triggerHTML = '<a href="#" class="dropdown-selector-trigger not-selected"><span class="dropdown-code">?</span><span class="fm icon-arrow-down-2 cssFontSize14"></span></a>';
		}
		this.$e.append(triggerHTML);
		this.$e.append('<span class="dropdown-options-gauntlet"></span>')
			.append('<span class="accessible-content" id="'+this.id+'_dialogInstructions">'+this.options.accessibility.dialogInstructionsText+'</span>')
			.append('<span class="accessible-content" id="'+this.id+'_firstItemInstructions">'+this.options.accessibility.firstItemInListText+'</span>')
			.append('<span class="accessible-content" id="'+this.id+'_lastItemInstructions">'+this.options.accessibility.lastItemInListText+'</span>');
		if(this.options.endlessLoop)
		{
			this.$e.find('#'+this.id+'_firstItemInstructions').append(' '+this.options.accessibility.firstItemEndlessText);
			this.$e.find('#'+this.id+'_lastItemInstructions').append(' '+this.options.accessibility.lastItemEndlessText);
		}
		this.$e.append('<span class="dropdown-options-list"></span>');
	},
	
	__registerHTML:			function()
	{
		this.$trigger	= this.$e.find('.dropdown-selector-trigger');
		this.$list		= this.$e.find('.dropdown-options-list');
		this.$gauntlet	= this.$e.find('.dropdown-options-gauntlet');
		this.$trigger.attr("aria-describedby",this.id+'_dialogInstructions');
	},
	
	__registerEvents:			function()
	{
		this.$trigger.bind('click',{o:this},function(e){ e.preventDefault(); e.data.o.activateSelection(); });
		this.$gauntlet.bind('click',{o:this},function(e){ e.data.o.deactivateSelection(); });
	}
});

$.extend(Q.PluginFactory.Dropdown.prototype,{
	activateSelection:		function()
	{
		this.$e.addClass('status-active');
		
		var p = {};
			p.canvasHeight	= this.$list.height();
			p.screenHeight	= $(document).height();
			p.canvasTop 	= this.$e.offset().top+this.$e.height();
		if(p.canvasHeight+p.canvasTop > p.screenHeight) { this.$list.addClass('top-aligned'); }
		else { this.$list.removeClass('top-aligned'); }
		
		if(this.options.data.length < 1 || arguments[0])
		{
			this.eventTrigger('onFirstActivate',{source:this.$trigger,instance:this});
		} else {
			this.$list.find('A.dropdown-option-item:first-child').focus();
		}
		this.eventTrigger('onActivate',{source:this.$trigger,instance:this});
	},
	
	deactivateSelection:	function()
	{
		this.eventTrigger('onClose',{instance:this});
		this.$e.removeClass('status-active');
	},
	
	acceptData:	function()
	{
		var data = (typeof(arguments[0])!=='undefined') ? arguments[0]:'No data provided by remote host.';
		if(typeof(data)==='string') { alert('Error: '+data); return false; }
		this.eventTrigger('onData',{instance:this,data:data});
		this.options.data = data;
		this.renderSelection();
		this.$list.find('A.dropdown-option-item:first-child').focus();
	},
	
	renderSelection:		function()
	{
		this.$list.empty();
		for(var i in this.options.data) { this.renderSelectionItem(this.options.data[i]); }
		this.$list.find('.dropdown-option-item:first-child').attr("aria-describedby",this.id+'_firstItemInstructions');
		this.$list.find('.dropdown-option-item:last-child').attr("aria-describedby",this.id+'_lastItemInstructions');
	},
	
	renderSelectionItem:	function(data)
	{
		var item; data.guid = Q.GUID();
		item = Q.Template(this.options.templates.listItemTemplate,data);
		if(!item)
		{
			item ='<a href="#" class="dropdown-option-item" id="'+data.guid+'"><span class="dropdown-label">'+ data.label +'</span></a>';
			Q.AddStack({message:'Defaulting to native output',type:'info',origin:this.name+'::renderSelectionItem()',arguments:arguments,instance:this.id});
		}
		this.$list.append(item);
		this.registerSelectionEvents({id:data.guid,label:data.label,meta:data.meta,selectedId:data.id});
	}
});

$.extend(Q.PluginFactory.Dropdown.prototype,{
	registerSelectionEvents:function(data)
	{
		Q.AddStack({message:'Registering events on item',type:'info',origin:this.name+'::registerSelectionEvents()',arguments:arguments,instance:this.id});
		this.$e.find('#'+data.id).bind('click',{o:this,data:data},function(e){ e.preventDefault(); e.data.o.selectItem(e.data.data); });
		this.$e.find('#'+data.id).bind('keydown',{o:this,data:data},function(e){ e.stopPropagation(); e.data.o.keyPressDropdown(e); });
	},
	
	selectItem:		function(data)
	{
		this.$trigger.find('.dropdown-label').html(data.label);
		this.selectedItem = {id:data.selectedId,label:data.label,meta:data.meta};
		this.eventTrigger('onSelect',{id:data.selectedId,label:data.label,meta:data.meta,instance:this});
		this.deactivateSelection();
		this.$trigger.focus();
	},
	
	keyPressDropdown:	function(e)
	{
		if(e.which===39||e.which===40) { this.keyActionNavigateNext(e); }
		if(e.which===37||e.which===38) { this.keyActionNavigatePrev(e); }
		if(e.which===27||e.which===19||e.which===35||e.which===8||e.which===9) { this.keyActionClose(e); }
	},
	
	scrollListTo:			function(nextEl)
	{
		var p = {};
		p.canvasHeight = this.$list.height();
		p.internalHeight = this.$list.get(0).scrollHeight;
		p.ratioCanvasVsInternal = p.canvasHeight/p.internalHeight;
		p.itemOffsetInCanvas = nextEl.position().top;
		p.itemOffsetInternal = nextEl.get(0).offsetTop;
		p.itemOffsetRatio = p.itemOffsetInternal * p.ratioCanvasVsInternal;

		var r = p.itemOffsetInternal - (p.canvasHeight*(p.itemOffsetInternal/ (p.internalHeight))) + (((p.itemOffsetRatio) * p.ratioCanvasVsInternal)/4);
		this.$list.scrollTop(r);
	},
	
	keyActionNavigateNext:	function(e)
	{
		e.preventDefault();
		var srcEl = $(e.target);
		var nextEl = (srcEl.is(':last-child')) ?
			(this.options.endlessLoop) ? this.$list.find('A.dropdown-option-item:first-child'):srcEl:
			srcEl.next('A.dropdown-option-item');
		$(nextEl).focus();
		this.scrollListTo($(nextEl));
	},
	
	keyActionNavigatePrev:	function(e)
	{
		e.preventDefault();
		var srcEl = $(e.target);
		var nextEl = (srcEl.is(':first-child')) ?
			(this.options.endlessLoop) ? this.$list.find('A.dropdown-option-item:last-child'):srcEl:
			srcEl.prev('A.dropdown-option-item');
		$(nextEl).focus();
		this.scrollListTo($(nextEl));
	},
	
	keyActionClose:			function(e)
	{
		e.preventDefault();
		this.deactivateSelection();
		this.$trigger.focus();
	}
});

/*
$.extend(Q.PluginFactory.Dropdown.prototype,{
	
});
*/