/*
 This file and all of its content are part of Q library
 by Jan Maska (http://janmaska.com), and published under
 Creative Commons CC BY-NC-SA License.
 For full license, go to https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode
 */

Q.Resource = Q.PluginFactory.Plugin();
$.extend(Q.Resource.prototype, Q.PluginFactory.Base.prototype);


$.extend(Q.Resource.prototype,{
	name:				'QResource',
	id:					false,
	options:			{},
	default_options:	{}
});

$.extend(true,Q.Resource.prototype,{
	eventQueue:
	{
		onActivate: 		[],
		onResourceInsert: 	[],
		onComplete: 		[]
	},

	onActivate: 			function() { if(typeof(arguments[0])=='undefined') {return this.eventQueue['onActivate'];} this.eventQueue.onActivate.push(arguments[0]); return this; },
	onResourceInsert: 		function() { if(typeof(arguments[0])=='undefined') {return this.eventQueue['onResourceInsert'];} this.eventQueue.onResourceInsert.push(arguments[0]); return this; },
	onComplete: 			function() { if(typeof(arguments[0])=='undefined') {return this.eventQueue['onComplete'];} this.eventQueue.onComplete.push(arguments[0]); return this; }
});

$.extend(Q.Resource.prototype,{
	resources: [],
	init: function(el,user_options)
	{	
		this.eventTrigger('onInit',{instance:this});
		this.add(this.options.files);
	}
});


$.extend(Q.Resource.prototype,{
	add: function()
	{
		/*
		Remember, this function can also be called externally,
		so we need to check whether or not an argument has been passed.
		And we need to make sure that the list of files to include is
		actually an array. I.e. transform a single string to a 1-element array.
		*/
		var files = (typeof(arguments[0])!='undefined')?arguments[0]:false;
		if(typeof(files)=='string') { files = [files]; }
		
		/* We're set up, cupcake! Let's fire the onActivate event! */
		this.eventTrigger('onActivate',{instance:this,files:files});
		
		/*
		Alright, cupcake! We're all set, let's loop over the list
		and add the files, one by one...
		*/
		for(var i=0;i<files.length;i++)
		{
			this.addResource(files[i]);
		}
		
		/* And all done! Let's fire the onComplete event! */
		this.eventTrigger('onComplete',{instance:this});
	},
	
	exists: function()
	{
		/* Aye, here's that check again. Just to make ABSOLUTELY SURE. */
		var file = (typeof(arguments[0])!='undefined') ? arguments[0]:false;
		if(!file) { return false; }
		
		for(i in this.resources) {if(this.resources[i].path == file) { return true; }}
		return false;
	},
	
	maybe: function()
	{
		/* Aye, here's that check again. Just to make ABSOLUTELY SURE. */
		var file = (typeof(arguments[0])!='undefined') ? arguments[0]:false;
		if(file){if(!this.exists(file)){this.add(file);}}
	},
	
	addResource: function()
	{
		/* Aye, here's that check again. Just to make ABSOLUTELY SURE. */
		var file = (typeof(arguments[0])!='undefined') ? arguments[0]:false;
		
		if(file)
		{
			/*
			Now, let's get the file type from the filename...
			*/
			var filetype = file.split('.');	filetype = filetype[filetype.length-1];
			
			/*
			Based on that, we can use several methods of inclusion:
			*/
			if(filetype =='css')		{ this.addCSSFile(file); }
			else if(filetype =='js')	{ $.getScript(file,function(data,textStatus){  }); }
			else { console.log('ERROR: "'+file+'" is not a valid resource!'); return; }
			
			/* We're set up, cupcake! Let's fire the onResourceInsert event! */
			this.eventTrigger('onResourceInsert',{instance:this,file:file,type:filetype});
			
			/*
			And we also list the loaded resource in an array of resources.
			This array might later be used to check against, and also
			for removing dynamically imported resources.
			*/
			this.resources.push({'path':file,'type':filetype});
			Q.AddStack({message:'Added:'+file,type:'Info',origin:this.name+'::addResource()',arguments:arguments});
		}
		
		return;
	},
	
	addCSSFile: function()
	{
		/* Aye, here's that check again. Just to make ABSOLUTELY SURE. */
		var file = (typeof(arguments[0])!='undefined') ? arguments[0]:false;
		
		if(file) {
			$('head').append('<link type="text/css" href="'+file+'" rel="stylesheet" />');
		}
	}
});

Q.PluginFactory.Register(Q.Resource);

/*

EXAMPLE USE:

Initialize the plugin:
$(document).QResource();

Load a single file:
$(document).QResource('add','path/to/my/file.css');

Load multiple files:
$(document).QResource('add',['path/to/my/file.css','path/to/my/otherFile.js]);

*/

/*

EXAMPLE USAGE OF REGISTERING AN EVENT HANDLER:
$(document).QResource();

$(document).QResource('onResourceInsert',
	function(){ alert("MQResource inserted my resource!"); }
);
// will throw the alert dialog once the 'addResource' function 
// has triggered "onResourceInsert" event.

*/